@@ -58,10 +58,11 @@ void ProBoundsArrayToPointerDecayCheck::registerMatchers(MatchFinder *Finder) {
   // 2) inside a range-for over an array
   // 3) if it converts a string literal to a pointer
   Finder->addMatcher(
-      implicitCastExpr(unless(hasParent(arraySubscriptExpr())),
-                       unless(hasParentIgnoringImpCasts(explicitCastExpr())),
-                       unless(isInsideOfRangeBeginEndStmt()),
-                       unless(hasSourceExpression(stringLiteral())))
+      implicitCastExpr(
+          unless(hasParent(arraySubscriptExpr())),
+          unless(hasParentIgnoringImpCasts(explicitCastExpr())),
+          unless(isInsideOfRangeBeginEndStmt()),
+          unless(hasSourceExpression(ignoringParens(stringLiteral()))))
           .bind("cast"),
       this);
 }
