@@ -15,14 +15,18 @@ namespace clang {
 namespace tidy {
 namespace fuchsia {
 
+namespace {
 AST_MATCHER(FunctionDecl, isFuchsiaOverloadedOperator) {
   if (const auto *CXXMethodNode = dyn_cast<CXXMethodDecl>(&Node)) {
     if (CXXMethodNode->isCopyAssignmentOperator() ||
         CXXMethodNode->isMoveAssignmentOperator())
       return false;
+    if (CXXMethodNode->getParent()->isLambda())
+      return false;
   }
   return Node.isOverloadedOperator();
 }
+} // namespace
 
 void OverloadedOperatorCheck::registerMatchers(MatchFinder *Finder) {
   Finder->addMatcher(functionDecl(isFuchsiaOverloadedOperator()).bind("decl"),
@@ -32,10 +36,10 @@ void OverloadedOperatorCheck::registerMatchers(MatchFinder *Finder) {
 void OverloadedOperatorCheck::check(const MatchFinder::MatchResult &Result) {
   const auto *D = Result.Nodes.getNodeAs<FunctionDecl>("decl");
   assert(D && "No FunctionDecl captured!");
-  
-  SourceLocation Loc = D->getLocStart();
+
+  SourceLocation Loc = D->getBeginLoc();
   if (Loc.isValid())
-    diag(Loc, "cannot overload %0") << D;
+    diag(Loc, "overloading %0 is disallowed") << D;
 }
 
 } // namespace fuchsia
