@@ -78,9 +78,9 @@ ForbiddenSubclassingCheck::ForbiddenSubclassingCheck(
 
 void ForbiddenSubclassingCheck::registerMatchers(MatchFinder *Finder) {
   // this check should only be applied to ObjC sources.
-  if (!getLangOpts().ObjC1 && !getLangOpts().ObjC2) {
+  if (!getLangOpts().ObjC)
     return;
-  }
+
   Finder->addMatcher(
       objcInterfaceDecl(
           isSubclassOf(
