@@ -34,7 +34,9 @@ public:
   void storeOptions(ClangTidyOptions::OptionMap &Options) override;
 
 private:
-    const std::vector<std::string> SpecialAcronyms;
+  const std::vector<std::string> SpecialAcronyms;
+  const bool IncludeDefaultAcronyms;
+  std::vector<std::string> EscapedAcronyms;
 };
 
 } // namespace objc
