@@ -14,21 +14,28 @@
 //===----------------------------------------------------------------------===//
 
 #include "ClangdLSPServer.h"
+#include "ClangdServer.h"
 #include "CodeComplete.h"
+#include <cstdio>
 #include <sstream>
 
+using namespace clang::clangd;
+
 extern "C" int LLVMFuzzerTestOneInput(uint8_t *data, size_t size) {
-  clang::clangd::JSONOutput Out(llvm::nulls(), llvm::nulls(), nullptr);
-  clang::clangd::CodeCompleteOptions CCOpts;
+  if (size == 0)
+    return 0;
+
+  // fmemopen isn't portable, but I think we only run the fuzzer on Linux.
+  std::FILE *In = fmemopen(data, size, "r");
+  auto Transport = newJSONTransport(In, llvm::nulls(),
+                                    /*InMirror=*/nullptr, /*Pretty=*/false,
+                                    /*Style=*/JSONStreamStyle::Standard);
+  CodeCompleteOptions CCOpts;
   CCOpts.EnableSnippets = false;
+  ClangdServer::Options Opts;
 
   // Initialize and run ClangdLSPServer.
-  clang::clangd::ClangdLSPServer LSPServer(
-      Out, clang::clangd::getDefaultAsyncThreadsCount(),
-      /*StorePreamblesInMemory=*/false, CCOpts, llvm::None, llvm::None,
-      /*BuildDynamicSymbolIndex=*/false);
-
-  std::istringstream In(std::string(reinterpret_cast<char *>(data), size));
-  LSPServer.run(In);
+  ClangdLSPServer LSPServer(*Transport, CCOpts, llvm::None, false, Opts);
+  LSPServer.run();
   return 0;
 }
